#include "LTCFunctions.h"

#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>



//////////////////////////////////////////////////////////////////////
//	Create CLTCGen object
//
//
//////////////////////////////////////////////////////////////////////

CLTCGen::CLTCGen()
{
	//Fill in initial values of the 80 bit smpte block
	smpte_block[0] = 0;
	smpte_block[1] = 0;
	smpte_block[2] = 0;
	smpte_block[3] = 0;
	smpte_block[4] = 0;
	smpte_block[5] = 0;
	smpte_block[6] = 0;
	smpte_block[7] = 0;
	smpte_block[8] = 0xFC;
	smpte_block[9] = 0xBF;
}


//////////////////////////////////////////////////////////////////////
// Cleanup CLTCGen object
//
//
//////////////////////////////////////////////////////////////////////

CLTCGen::~CLTCGen()
{
	delete [] mWav0Pos;
	delete [] mWav0Neg;
	delete [] mWav0PosEx;
	delete [] mWav0NegEx;

	delete [] mWav1Pos;
	delete [] mWav1Neg;
	delete [] mWav1PosEx;
	delete [] mWav1NegEx;
}


//////////////////////////////////////////////////////////////////////
// Initialize CLTCGen object
//
//
//////////////////////////////////////////////////////////////////////

bool CLTCGen::Init(DVD_TIMECODE *DVDTimeCodeStart, int SampleRate,
		int ChannelCount, int FPS, bool DropFrame, double Amplify, bool Square)
{
	// Initialize the timecode structure that will hold the current time
	m_TimeCode = 0;
	m_DVDTimeCode = (DVD_TIMECODE *)&m_TimeCode;

	// Initialize the timecode structure that holds the timecode start value
	m_TimeCodeStart = 0;
	m_DVDTimeCodeStart = (DVD_TIMECODE *)&m_TimeCodeStart;
	memcpy(m_DVDTimeCodeStart, DVDTimeCodeStart, sizeof(DVDTimeCodeStart));

	// If true, generate a square waveform
	bSquare = Square;

	// Initialize audio and video properties
	m_iSampleRate = SampleRate;
	m_iChannelCount = ChannelCount;
	m_iFPS = FPS;
	bDropFrame = DropFrame;
	m_dAmplify = Amplify;

	// The number of audio samples per video frame
	m_iFrameDur = m_iSampleRate / m_iFPS;

	// The number of audio samples per bit of the smpte_block
	m_iBitDur = m_iFrameDur / 80;

	// If m_iFrameDur is not evenly divisble by 80,
	// this value will contain the remainder
	// which is the number of extra audio samples that must
	// be generated for each smpte_block
	m_iExtraSamples = m_iFrameDur % 80;

	// Number of audio samples for the rise duration
	// Set to 0 if creating a square wave, otherwise set to 25 s (microseconds)
	if (bSquare)
		m_iRiseDur = 0;
	else
		m_iRiseDur = 25 * m_iSampleRate / 1000000.0;

	// Current amplitude of a sample, initialize to the max value
	m_iAmplitude = MAX_AMPLITUDE * m_dAmplify;
	
	// Max amplitude of a sample
	m_iMaxAmplitude = MAX_AMPLITUDE * m_dAmplify;

	// Initialze the random number generator
	// This is used to add +- 5% of m_iMaxAmplitude to each audio sample
	srand( (unsigned)time( NULL ) );

	InitWaveForms();

	return true;
}


//////////////////////////////////////////////////////////////////////
// Initialize CLTCGen object
//
//
//////////////////////////////////////////////////////////////////////

bool CLTCGen::InitWaveForms()
{
	// The current sample position
	int m_iSamplePos = 0;

	// The random number and it's phase
	// Used to help create a non-square waveform
	int m_iRand = 0;
	int m_iRandPhase = 1;

	// How many samples to generate for the rise and fall
	int m_iRiseCount = m_iRiseDur;

	int m_iAmplitude0 = 0;
	int m_iAmplitude1 = 0;

	// The standard waveforms for bit = 0
	mWav0Pos = new short[m_iBitDur * m_iChannelCount];
	mWav0Neg = new short[m_iBitDur * m_iChannelCount];

	// The extended waveforms for bit = 0
	// Only needed if (m_iExtraSamples = m_iFrameDur % 80) != 0
	mWav0PosEx = new short[(m_iBitDur + 1) * m_iChannelCount];
	mWav0NegEx = new short[(m_iBitDur + 1) * m_iChannelCount];

	// The standard waveforms for bit = 1
	mWav1Pos = new short[m_iBitDur * m_iChannelCount];
	mWav1Neg = new short[m_iBitDur * m_iChannelCount];

	// The extended waveforms for bit = 0
	// Only needed if (m_iExtraSamples = m_iFrameDur % 80) != 0
	mWav1PosEx = new short[(m_iBitDur + 1) * m_iChannelCount];
	mWav1NegEx = new short[(m_iBitDur + 1) * m_iChannelCount];

	// Create the 8 different types of waveforms
	for (int h = 0; h < m_iBitDur - m_iRiseDur; h++) {
		//if not a square waveform, create a randomness of +-5% of m_iMaxAmplitude
		if (!bSquare) {
			m_iRand = rand();
			if (m_iRand * 2 < RAND_MAX)
				m_iRand += RAND_MAX / 2;
			m_iRand = m_iRandPhase * m_iMaxAmplitude * 0.05 * ((float)m_iRand / (float)RAND_MAX);
			m_iRandPhase = -m_iRandPhase;
		}

		mWav0Pos[m_iSamplePos] = m_iAmplitude + m_iRand;
		mWav0Neg[m_iSamplePos] = -m_iAmplitude - m_iRand;
		mWav0PosEx[m_iSamplePos] = m_iAmplitude + m_iRand;
		mWav0NegEx[m_iSamplePos] = -m_iAmplitude - m_iRand;

		if (m_iChannelCount == 2) {
			mWav0Pos[m_iSamplePos+1] = m_iAmplitude + m_iRand;
			mWav0Neg[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
			mWav0PosEx[m_iSamplePos+1] = m_iAmplitude + m_iRand;
			mWav0NegEx[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
		}

		// When h has reached the halfway point, switch phase for bit == 1
		if (h < m_iBitDur / 2) {
			mWav1Pos[m_iSamplePos] = m_iAmplitude + m_iRand;
			mWav1Neg[m_iSamplePos] = -m_iAmplitude - m_iRand;
			mWav1PosEx[m_iSamplePos] = m_iAmplitude + m_iRand;
			mWav1NegEx[m_iSamplePos] = -m_iAmplitude - m_iRand;

			if (m_iChannelCount == 2) {
				mWav1Pos[m_iSamplePos+1] = m_iAmplitude + m_iRand;
				mWav1Neg[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
				mWav1PosEx[m_iSamplePos+1] = m_iAmplitude + m_iRand;
				mWav1NegEx[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
			}
		} else {
			mWav1Pos[m_iSamplePos] = -m_iAmplitude - m_iRand;
			mWav1Neg[m_iSamplePos] = m_iAmplitude + m_iRand;
			mWav1PosEx[m_iSamplePos] = -m_iAmplitude - m_iRand;
			mWav1NegEx[m_iSamplePos] = m_iAmplitude + m_iRand;

			if (m_iChannelCount == 2) {
				mWav1Pos[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
				mWav1Neg[m_iSamplePos+1] = m_iAmplitude + m_iRand;
				mWav1PosEx[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
				mWav1NegEx[m_iSamplePos+1] = m_iAmplitude + m_iRand;
			}
		}

		if (m_iChannelCount == 2)
			m_iSamplePos += 2;
		else
			m_iSamplePos++;
	}

	// if (m_iRiseDur == 0)
		// Add one extra sample for the extended waveforms
	// if (m_iRiseDur =! 0)
		// the extra sample will be added to the end of the rise data
	if (m_iRiseDur == 0) {
		mWav0PosEx[m_iSamplePos] = m_iAmplitude + m_iRand;
		mWav0NegEx[m_iSamplePos] = -m_iAmplitude - m_iRand;
		mWav1PosEx[m_iSamplePos] = -m_iAmplitude - m_iRand;
		mWav1NegEx[m_iSamplePos] = m_iAmplitude + m_iRand;

		if (m_iChannelCount == 2) {
			mWav0PosEx[m_iSamplePos+1] = m_iAmplitude + m_iRand;
			mWav0NegEx[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
			mWav1PosEx[m_iSamplePos+1] = -m_iAmplitude - m_iRand;
			mWav1NegEx[m_iSamplePos+1] = m_iAmplitude + m_iRand;
		}
	} else {
		// create the rise and fall for the standard waveforms

		// Get the current sample position so it can be restored
		int m_iSamplePosHold = m_iSamplePos;

		for (int h = 0; h < m_iRiseCount; h++) {
			// Increment and Decrement the sample to create the rise and fall
			m_iAmplitude0 = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));
			m_iAmplitude1 = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));

			//if not a square waveform, create a randomness of +-5% of m_iMaxAmplitude
			if (!bSquare) {
				m_iRand = rand();
				if (m_iRand * 2 < RAND_MAX)
					m_iRand += RAND_MAX / 2;
				m_iRand = m_iRandPhase * m_iMaxAmplitude * 0.05 * ((float)m_iRand / (float)RAND_MAX);
				m_iRandPhase = -m_iRandPhase;
			}

			m_iRand = 0;

			mWav0Pos[m_iSamplePos] = m_iAmplitude0 + m_iRand;
			mWav0Neg[m_iSamplePos] = -m_iAmplitude0 - m_iRand;

			mWav1Pos[m_iSamplePos] = -m_iAmplitude1 - m_iRand;
			mWav1Neg[m_iSamplePos] = m_iAmplitude1 + m_iRand;

			if (m_iChannelCount == 2) {
				mWav0Pos[m_iSamplePos+1] = m_iAmplitude0 + m_iRand;
				mWav0Neg[m_iSamplePos+1] = -m_iAmplitude0 - m_iRand;

				mWav1Pos[m_iSamplePos+1] = -m_iAmplitude1 - m_iRand;
				mWav1Neg[m_iSamplePos+1] = m_iAmplitude1 + m_iRand;
			}

			if (m_iChannelCount == 2)
				m_iSamplePos += 2;
			else
				m_iSamplePos++;
		}

		// Reset the sample postion
		m_iSamplePos = m_iSamplePosHold;

		m_iRiseCount++;

		// create the rise and fall for the Ex waveforms
		for (int h = 0; h < m_iRiseCount; h++) {
			// Increment and Decrement the sample to create the rise and fall
			m_iAmplitude0 = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));
			m_iAmplitude1 = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));

			//if not a square waveform, create a randomness of +-5% of m_iMaxAmplitude
			if (!bSquare) {
				m_iRand = rand();
				if (m_iRand * 2 < RAND_MAX)
					m_iRand += RAND_MAX / 2;
				m_iRand = m_iRandPhase * m_iMaxAmplitude * 0.05 * ((float)m_iRand / (float)RAND_MAX);
				m_iRandPhase = -m_iRandPhase;
			}

			m_iRand = 0;

			mWav0PosEx[m_iSamplePos] = m_iAmplitude0 + m_iRand;
			mWav0NegEx[m_iSamplePos] = -m_iAmplitude0 - m_iRand;

			mWav1PosEx[m_iSamplePos] = -m_iAmplitude1 - m_iRand;
			mWav1NegEx[m_iSamplePos] = m_iAmplitude1 + m_iRand;

			if (m_iChannelCount == 2) {
				mWav0PosEx[m_iSamplePos+1] = m_iAmplitude0 + m_iRand;
				mWav0NegEx[m_iSamplePos+1] = -m_iAmplitude0 - m_iRand;

				mWav1PosEx[m_iSamplePos+1] = -m_iAmplitude1 - m_iRand;
				mWav1NegEx[m_iSamplePos+1] = m_iAmplitude1 + m_iRand;
			}

			if (m_iChannelCount == 2)
				m_iSamplePos += 2;
			else
				m_iSamplePos++;
		}
	}

	return true;
}


//////////////////////////////////////////////////////////////////////
//	Obsolete function
//
//
//////////////////////////////////////////////////////////////////////

unsigned int CLTCGen::Generate(short *sample, unsigned int sampleStart, unsigned int numSamples)
{
	unsigned int m_iCount = 0;
	int m_iAmplitude = MAX_AMPLITUDE * m_dAmplify;
	int m_iExtraSampleCount = 0;

	LTC_SampleNum_to_BCD(sampleStart, m_iSampleRate, m_DVDTimeCode, m_DVDTimeCodeStart, bDropFrame);
	LTC_EncodeTime(smpte_block, m_DVDTimeCode, bDropFrame);
	LTC_Parity(smpte_block, m_iFPS);

	for (int Byte = 0; Byte < 10; Byte++) {
		for (int bit = 0; bit < 8; bit++) {
			m_iAmplitude = -m_iAmplitude;
			for (int h = 0; h < m_iBitDur; h++) {
				if ( (smpte_block[Byte] & (1 << bit)) && (h == m_iBitDur / 2) )
					m_iAmplitude = -m_iAmplitude;

				sample[m_iCount] = m_iAmplitude;
				m_iCount++;

				if (m_iChannelCount == 2) {
					sample[m_iCount] = m_iAmplitude;
					m_iCount++;
				}
			}

			if (m_iExtraSampleCount < m_iExtraSamples) {
				if (m_iExtraSampleCount > 40 || (m_iExtraSampleCount < 40 && (Byte*bit) % 2 == 0) ) {
					sample[m_iCount] = m_iAmplitude;
					m_iCount++;

					if (m_iChannelCount == 2) {
						sample[m_iCount] = m_iAmplitude;
						m_iCount++;
					}

					m_iExtraSampleCount++;
				}
			}
		}
	}

	return m_iCount;
}


//////////////////////////////////////////////////////////////////////
// Obsolete function
//
// Generate the audio data and return the number of audio samples
//   actually created
//
// sampleStart = the sample number to start calculating from
// numSamples = the number of samples to create multiplied
//    by the number of audio channels
//////////////////////////////////////////////////////////////////////

unsigned int CLTCGen::Generate2(short *sample, unsigned int sampleStart, unsigned int numSamples)
{
	int m_iLoopCount;
	unsigned int frameAligned_sampleStart;
	unsigned int m_iCount = 0;
	unsigned int m_iSamplePos = 0;
	int m_iExtraSampleCount = 0;

	int m_iRand = 0;
	int m_iRandPhase = 1;

	unsigned int m_iOffset = 0;

	// This is the number of frames we have been requested to generate
	// Always round up so there is enough data created
	m_iLoopCount = ceil( (1.0 / m_iChannelCount) * numSamples / m_iFrameDur);

	// If (numSamples/2) is smaller than m_iFrameDur, then m_iLoopCount
	// would equal zero, in this case we set the loop count to 1
	if (m_iLoopCount == 0)
		m_iLoopCount = 1;

	// Check to see if sampleStart is frame aligned
	// Data is frame aligned if (sampleStart % m_iFrameDur == 0)
	// If m_iOffset > 0, we will generate the data but will not copy
	// it back to the wav file
	m_iOffset = LTC_SampleNum_to_BCD(sampleStart, m_iSampleRate,
				m_DVDTimeCode, m_DVDTimeCodeStart, bDropFrame);

	// Frame align the starting sample number
	if (m_iOffset)
		frameAligned_sampleStart = sampleStart - m_iOffset;
	else
		frameAligned_sampleStart = sampleStart;


	// Create data for m_iLoopCount number of frames.
	// Each loop, increase the frameAligned_sampleStart value by the duration of 1 video frame
	for (int i = 0; i < m_iLoopCount; i++, frameAligned_sampleStart += m_iFrameDur) {		

		// Convert a sample number to a BCD Timecode structure
		LTC_SampleNum_to_BCD(frameAligned_sampleStart, m_iSampleRate,
				m_DVDTimeCode, m_DVDTimeCodeStart, bDropFrame);

		// Encode a BCD Timecode structure to the smpte_block
		LTC_EncodeTime(smpte_block, m_DVDTimeCode, bDropFrame);

		// Add parity to the smpte_block
		LTC_Parity(smpte_block, m_iFPS);

		// Initialize the extra bit counter
		m_iExtraSampleCount = 0;

		// Loop through the smpte_block one byte at a time
		for (int mByte = 0; mByte < 10; mByte++) {
			// Loop through each bit
			for (int mBit = 0; mBit < 8; mBit++) {
				// phase invert the max amplitude of the waveform
				m_iMaxAmplitude = -m_iMaxAmplitude;

				// set the current amplitude to the maximum value
				m_iAmplitude = m_iMaxAmplitude;

				// Loop through
				for (int h = 0; h < m_iBitDur - m_iRiseDur; h++) {
					if (m_iSamplePos >= numSamples) {
					//if (m_iCount * m_iChannelCount >= numSamples) {
						mBit = 100;
						mByte = 100;
						i = m_iLoopCount;
						break;
					}

					// When h has reached the halfway point, switch phase for bit == 1
					if ( (smpte_block[mByte] & (1 << mBit)) && (h == m_iBitDur / 2) ) {
						m_iMaxAmplitude = -m_iMaxAmplitude;
						m_iAmplitude = m_iMaxAmplitude;
					}

					//if not a square waveform, create a randomness of +-5% of m_iMaxAmplitude
					if (!bSquare) {
						m_iRand = rand();
						if (m_iRand * 2 < RAND_MAX)
							m_iRand += RAND_MAX / 2;
						m_iRand = m_iRandPhase * m_iMaxAmplitude * 0.05 * ((float)m_iRand / (float)RAND_MAX);
						m_iRandPhase = -m_iRandPhase;
					}

					if (m_iCount >= m_iOffset) {
						sample[m_iSamplePos++] = m_iAmplitude + m_iRand;

						if (m_iChannelCount == 2) {
							sample[m_iSamplePos++] = m_iAmplitude + m_iRand;
						}
					}
					m_iCount++;
				}

				if (m_iRiseDur == 0) {
					if (m_iExtraSampleCount < m_iExtraSamples && m_iCount >= m_iOffset) {
						if (m_iExtraSampleCount > 40 || (m_iExtraSampleCount < 40 && (mByte*mBit) % 2 == 0) ) {
							sample[m_iSamplePos++] = m_iAmplitude;

							if (m_iChannelCount == 2) {
								sample[m_iSamplePos++] = m_iAmplitude;
							}

							m_iExtraSampleCount++;
						}
					}
				}

				int m_iRiseCount = m_iRiseDur;

				if (!bSquare && m_iExtraSampleCount < m_iExtraSamples 
						&& (m_iExtraSampleCount > 40 || (m_iExtraSampleCount < 40 && (mByte*mBit) % 2 == 0)) ) {
					m_iRiseCount++;
					m_iExtraSampleCount++;
				}

				for (int h = 0; h < m_iRiseCount; h++) {
					if (m_iSamplePos >= numSamples) {
						mBit = 100;
						mByte = 100;
						i = m_iLoopCount;
						break;
					}

					if (m_iMaxAmplitude > 0) {
						m_iAmplitude = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));
					} else {
						m_iAmplitude = m_iMaxAmplitude - (1.0 * m_iMaxAmplitude * float(h+1) / float(m_iRiseCount+1));
					}

					//if not a square waveform, create a randomness of +-5% of m_iMaxAmplitude
					if (!bSquare) {
						m_iRand = rand();
						if (m_iRand * 2 < RAND_MAX)
							m_iRand += RAND_MAX / 2;
						m_iRand = m_iRandPhase * m_iMaxAmplitude * 0.05 * ((float)m_iRand / (float)RAND_MAX);
						m_iRandPhase = -m_iRandPhase;
					}

					if (m_iCount >= m_iOffset) {
						sample[m_iSamplePos++] = m_iAmplitude + m_iRand;

						if (m_iChannelCount == 2) {
							sample[m_iSamplePos++] = m_iAmplitude + m_iRand;
						}
					}
					m_iCount++;
				}
			} // for mByte < 10

			// Only use m_iOffset data during the first loop 
			if (m_iOffset)
				m_iOffset = 0;
		} // for i < m_iLoopCount
	}

	return m_iSamplePos;
}


//////////////////////////////////////////////////////////////////////
// Generate the audio data and return the number of audio samples
//   actually created
//
// sampleStart = the sample number to start calculating from
// numSamples = the number of samples to create multiplied
//    by the number of audio channels
//////////////////////////////////////////////////////////////////////

unsigned int CLTCGen::Generate3(short *sample, unsigned int sampleStart, unsigned int numSamples)
{
	int m_iLoopCount;
	unsigned int frameAligned_sampleStart;
	unsigned int m_iOverallPos = 0;
	unsigned int m_iSamplePos = 0;
	int m_iSampleIncrement = 0;
	int m_iExtraSampleCount = 0;
	bool bExtraSample = false;

	int m_iRand = 0;
	int m_iRandPhase = 1;

	unsigned int m_iOffset = 0;

	// This is the number of frames we have been requested to generate
	// Always round up so there is enough data created
	m_iLoopCount = ceil( (1.0 / m_iChannelCount) * numSamples / m_iFrameDur);

	// If (numSamples/2) is smaller than m_iFrameDur, then m_iLoopCount
	// would equal zero, in this case we set the loop count to 1
	if (m_iLoopCount == 0)
		m_iLoopCount = 1;

	// Check to see if sampleStart is frame aligned
	// Data is frame aligned if (sampleStart % m_iFrameDur == 0)
	// If m_iOffset > 0, we will generate the data but will not copy
	// it back to the wav file
	m_iOffset = LTC_SampleNum_to_BCD(sampleStart, m_iSampleRate,
				m_DVDTimeCode, m_DVDTimeCodeStart, bDropFrame);

	// Frame align the starting sample number
	if (m_iOffset)
		frameAligned_sampleStart = sampleStart - m_iOffset;
	else
		frameAligned_sampleStart = sampleStart;

	// Create data for m_iLoopCount number of frames.
	// A loop creats the entire waveform for one frame of video data
	// Each loop, increase the frameAligned_sampleStart value by the duration of 1 video frame
	for (int i = 0; i < m_iLoopCount; i++, frameAligned_sampleStart += m_iFrameDur) {		

		// Convert a sample number to a BCD Timecode structure
		LTC_SampleNum_to_BCD(frameAligned_sampleStart, m_iSampleRate,
				m_DVDTimeCode, m_DVDTimeCodeStart, bDropFrame);

		// Encode a BCD Timecode structure into the smpte_block
		LTC_EncodeTime(smpte_block, m_DVDTimeCode, bDropFrame);

		// Add parity to the smpte_block
		LTC_Parity(smpte_block, m_iFPS);

		// Initialize the extra sample counter
		m_iExtraSampleCount = 0;

		// Loop through the smpte_block one byte at a time
		for (int mByte = 0; mByte < 10; mByte++) {
			// Loop through each bit
			for (int mBit = 0; mBit < 8; mBit++) {
				// phase invert the max amplitude
				m_iMaxAmplitude = -m_iMaxAmplitude;

				// Calculate the amount to increase our sample counters by
				if ( m_iExtraSampleCount < m_iExtraSamples // && m_iOverallPos >= m_iOffset) {
						&& (m_iExtraSampleCount > 40 || (m_iExtraSampleCount < 40 && (mByte*mBit) % 2 == 0)) ) { /*mByte**/
					m_iSampleIncrement = (m_iBitDur + 1) * m_iChannelCount;
					m_iExtraSampleCount++;
					bExtraSample = true;
				} else {
					m_iSampleIncrement = m_iBitDur * m_iChannelCount;
					bExtraSample = false;
				}

				if (m_iSamplePos >= numSamples) {
					// Check to make sure that we are not returning more samples than requested
					// Exit the function if we have
					mBit = 100;
					mByte = 100;
					i = m_iLoopCount;

					// need a fullframe param
					// do not switch phase if did not send out full frame
					// ???
					// I think this might be causing some of the problems
					break;
				} else if (m_iSamplePos + m_iSampleIncrement > numSamples) {
					// Check to make sure there is enough room to add an entire video frame
					// adjust copy parameters if there is only enough room for a partial frame

					m_iSampleIncrement -= (numSamples - m_iSamplePos);
				}

				// if there is a sample offset, this means we do not want to return
				// the entire frame.  Check to see if we have reached the offset
				// before copying data into our output sample
				if ( m_iOffset > 0 && m_iOverallPos < m_iOffset + m_iSampleIncrement / m_iChannelCount) {
					// increment the overall pos counter, do not copy data and continue
					m_iOverallPos += m_iSampleIncrement / m_iChannelCount;
					continue;
				} else if ( m_iOffset > 0 && (m_iOverallPos >= m_iOffset + m_iSampleIncrement / m_iChannelCount) ){
					// copy partial data, reset m_iOffset to 0 so we do not arrive here again
					// and continue
					int m_iPartialIncrement = m_iChannelCount * (m_iOffset - m_iOverallPos);

					//Copy a partial video frame to our output sample
					if (smpte_block[mByte] & (1 << mBit)) {
						// Current bit == 1
						if (bExtraSample) {
							// Use WavEx which contains one extra sample to copy the wavform
							if (m_iMaxAmplitude > 0) {
								memcpy(&sample[m_iSamplePos], &mWav1PosEx[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							} else {
								memcpy(&sample[m_iSamplePos], &mWav1NegEx[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							}
						} else {
							// Use the standard wavform
							if (m_iMaxAmplitude > 0) {
								memcpy(&sample[m_iSamplePos], &mWav1Pos[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							} else {
								memcpy(&sample[m_iSamplePos], &mWav1Neg[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							}
						}

						// phase invert the max amplitude
						m_iMaxAmplitude = -m_iMaxAmplitude;
					} else if (m_iOffset == 0) {
						// Current bit == 0
						if (bExtraSample) {
							// Use WavEx which contains one extra sample to copy the wavform
							if (m_iMaxAmplitude > 0) {
								memcpy(&sample[m_iSamplePos], &mWav0PosEx[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							} else {
								memcpy(&sample[m_iSamplePos], &mWav0NegEx[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							}
						} else {
							// Use the standard wavform
							if (m_iMaxAmplitude > 0) {
								memcpy(&sample[m_iSamplePos], &mWav0Pos[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							} else {
								memcpy(&sample[m_iSamplePos], &mWav0Neg[m_iPartialIncrement], 2*(m_iSampleIncrement-m_iPartialIncrement));
							}
						}
					}

					// Increment the sample pos counters
					m_iSamplePos += m_iPartialIncrement;
					m_iOverallPos += m_iPartialIncrement / m_iChannelCount;

					m_iOffset = 0;
					continue;
				}

				//Copy an entire video frame to our output sample
				if (smpte_block[mByte] & (1 << mBit)) {
					// Current bit == 1
					if (bExtraSample) {
						// Use WavEx which contains one extra sample to copy the wavform
						if (m_iMaxAmplitude > 0) {
							memcpy(&sample[m_iSamplePos], mWav1PosEx, 2*m_iSampleIncrement);
						} else {
							memcpy(&sample[m_iSamplePos], mWav1NegEx, 2*m_iSampleIncrement);
						}
					} else {
						// Use the standard wavform
						if (m_iMaxAmplitude > 0) {
							memcpy(&sample[m_iSamplePos], mWav1Pos, 2*m_iSampleIncrement);
						} else {
							memcpy(&sample[m_iSamplePos], mWav1Neg, 2*m_iSampleIncrement);
						}
					}

					// phase invert the max amplitude
					m_iMaxAmplitude = -m_iMaxAmplitude;
				} else {
					// Current bit == 0
					if (bExtraSample) {
						// Use WavEx which contains one extra sample to copy the wavform
						if (m_iMaxAmplitude > 0) {
							memcpy(&sample[m_iSamplePos], mWav0PosEx, 2*m_iSampleIncrement);
						} else {
							memcpy(&sample[m_iSamplePos], mWav0NegEx, 2*m_iSampleIncrement);
						}
					} else {
						// Use the standard wavform
						if (m_iMaxAmplitude > 0) {
							memcpy(&sample[m_iSamplePos], mWav0Pos, 2*m_iSampleIncrement);
						} else {
							memcpy(&sample[m_iSamplePos], mWav0Neg, 2*m_iSampleIncrement);
						}
					}
				}

				// Increment the sample pos counters
				m_iSamplePos += m_iSampleIncrement;
				m_iOverallPos += m_iSampleIncrement / m_iChannelCount;
			} // for mBit < 8
		} // for for mByte < 10
	} // for i < m_iLoopCount

	// return the number of samples created
	return m_iSamplePos;
}

//////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////

void CLTCGen::LTC_EncodeTime(unsigned char *block, DVD_TIMECODE *TimePos, bool dropframe)
{
	block[0] = (block[0] & 0xF0) | TimePos->Frames1;
	block[1] = (block[1] & 0xFC) | TimePos->Frames10;
	block[2] = (block[2] & 0xF0) | TimePos->Seconds1;
	block[3] = (block[3] & 0xF8) | TimePos->Seconds10;
	block[4] = (block[4] & 0xF0) | TimePos->Minutes1;
	block[5] = (block[5] & 0xF8) | TimePos->Minutes10;
	block[6] = (block[6] & 0xF0) | TimePos->Hours1;
	block[7] = (block[7] & 0xFC) | TimePos->Hours10;

	if (dropframe)
		block[1] |= 0x04;
}

//////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////

void CLTCGen::LTC_UserBits(unsigned char *block, unsigned int userbits)
{
}

//////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////

void CLTCGen::LTC_Parity(unsigned char *block, int frame_rate)
{
	int count = 0;
	for (int i = 0; i < 10; i++) {
		for (int b = 0; b < 8; b++) {
			if (block[i] & (1 << b) )
				count++;
		}
	}

	if (count % 2 != 0 && (block[3] & 0x08) == 0x08)
		block[3] &= ~0x08;
	else if (count % 2 != 0 && (block[3] & 0x08) == 0)
		block[3] |= 0x08;
}

//////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////

unsigned int CLTCGen::LTC_SampleNum_to_BCD(unsigned int SampleNum, int SampleRate, DVD_TIMECODE *TimePos, DVD_TIMECODE *TimeOffset, bool DropFrame)
{
	unsigned int m_iOffset = 0;
	unsigned int m_iTotalFrames;
	double m_dTotalFrames;
	int m_iHours;
	int m_iMinutes;
	int m_iSeconds;
	int m_iFrames;

	m_dTotalFrames = 1.0 * m_iFPS * double(SampleNum) / double(SampleRate);
	m_iTotalFrames = floor(1.0 * m_iFPS * double(SampleNum) / double(SampleRate));

	/*if (DropFrame) {
		m_iTotalFrames += 2*(int(m_iTotalFrames / 1798) - int(m_iTotalFrames / 17980));
		m_dTotalFrames += 2*(int(m_iTotalFrames / 1798) - int(m_iTotalFrames / 17980));
	}*/

	m_iOffset = (1.0 * m_dTotalFrames * SampleRate) / (1.0 * m_iFPS)
		- (1.0 * m_iTotalFrames * SampleRate) / (1.0 * m_iFPS);

	/*if (DropFrame) {
		m_iMinutes = (m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS);
		m_iHours = m_iTotalFrames / (60 * 60 * m_iFPS);

		m_iTotalFrames +=
			2 * ((m_iHours * 60 + m_iMinutes) - (((m_iHours * 60) + m_iMinutes) / 10));
	}*/

	if (DropFrame) {
		//m_iFrames = m_iTotalFrames % m_iFPS;
		//m_iSeconds = unsigned int((m_iTotalFrames % (60 * m_iFPS)) / m_iFPS);
		//m_iMinutes = unsigned int((m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS));
		//m_iHours = unsigned int(m_iTotalFrames / (60 * 60 * m_iFPS));
		
		//m_iFrames = ( fmod( m_iTotalFrames, 29.97 ));
		//m_iSeconds = (( fmod(m_iTotalFrames, (60 * 29.97)) ) / 29.97);
		m_iMinutes = (( fmod(m_iTotalFrames, (60 * 60 * 29.97)) ) / (60 * 29.97));
		m_iHours = (m_iTotalFrames / (60 * 60 * 29.97));

		m_iTotalFrames +=
			2 * ((m_iHours * 60 + m_iMinutes) - (((m_iHours * 60) + m_iMinutes) / 10));

		m_iFrames = m_iTotalFrames % m_iFPS;
		m_iSeconds = ((m_iTotalFrames % (60 * m_iFPS)) / m_iFPS);
		m_iMinutes = ((m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS));
		//m_iHours = (m_iTotalFrames / (60 * 60 * m_iFPS));
		if ( (m_iFrames == 0 || m_iFrames == 1) & (m_iSeconds == 0) & (m_iMinutes % 10 > 0) ) {
			m_iTotalFrames = m_iTotalFrames - 2;
			/*m_iFrames = m_iTotalFrames % m_iFPS;
			m_iSeconds = unsigned int((m_iTotalFrames % (60 * m_iFPS)) / m_iFPS);
			m_iMinutes = unsigned int((m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS));
			m_iHours = unsigned int(m_iTotalFrames / (60 * 60 * m_iFPS));*/
		}
	}

	m_iTotalFrames += (TimeOffset->Hours10 * 10 * 60 * 60 * m_iFPS)
			+ (TimeOffset->Hours1 * 60 * 60 * m_iFPS)
			+ (TimeOffset->Minutes10 * 10 * 60 * m_iFPS)
			+ (TimeOffset->Minutes1 * 60 * m_iFPS)
			+ (TimeOffset->Seconds10 * 10 * m_iFPS)
			+ (TimeOffset->Seconds1 * m_iFPS)
			+ (TimeOffset->Frames10 * 10)
			+ (TimeOffset->Frames1);

	/*m_iFrames = m_iTotalFrames % m_iFPS;
	m_iSeconds =  (m_iTotalFrames % (60 * m_iFPS)) / m_iFPS;
	m_iMinutes = (m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS);
	m_iHours = m_iTotalFrames / (60 * 60 * m_iFPS);*/

	m_iFrames = m_iTotalFrames % m_iFPS;
	m_iSeconds = unsigned int((m_iTotalFrames % (60 * m_iFPS)) / m_iFPS);
	m_iMinutes = unsigned int((m_iTotalFrames % (60 * 60 * m_iFPS)) / (60 * m_iFPS));
	m_iHours = unsigned int(m_iTotalFrames / (60 * 60 * m_iFPS));

	/*if ( (DropFrame && (m_iMinutes % 10 != 0) && m_iSeconds == 0)
		&& (m_iFrames == 0 || m_iFrames == 1) ) {
			m_iFrames += 2;
	}*/

	//Do not allow timecode greater than 23:59:59.999
	if (m_iHours > 23)
		m_iHours = m_iHours % 24;

	TimePos->Hours10 = m_iHours / 10;
	TimePos->Hours1  = m_iHours % 10;
	TimePos->Minutes10 = m_iMinutes / 10;
	TimePos->Minutes1  = m_iMinutes % 10;
	TimePos->Seconds10 = m_iSeconds / 10;
	TimePos->Seconds1  = m_iSeconds % 10;
	TimePos->Frames10 = m_iFrames / 10;
	TimePos->Frames1  = m_iFrames % 10;

	/*FILE *stream = fopen("test.txt", "ab");
	fprintf(stream, "%d%d:%d%d:%d%d:%d%d\n",
		TimePos->Hours10,
		TimePos->Hours1,
		TimePos->Minutes10,
		TimePos->Minutes1,
		TimePos->Seconds10,
		TimePos->Seconds1,
		TimePos->Frames10,
		TimePos->Frames1);
	fclose(stream);*/

	return m_iOffset;
}